import { NestFactory } from '@nestjs/core';
import { BadRequestException, ValidationError, ValidationPipe } from '@nestjs/common';
import { SwaggerModule, DocumentBuilder } from '@nestjs/swagger';
import { AppModule } from './app.module';
import { AlbumModule } from './album/album.module';
import { SongModule } from './song/song.module';
import { ConfigService } from '@nestjs/config';

async function bootstrap() {
  const app = await NestFactory.create(AppModule, {
    logger: ['error', 'warn'],
  });
  const configService = app.get<ConfigService>(ConfigService);
  app.enableCors({
    origin: [configService.get<string>('app.frontend_url')],
    methods: 'GET,HEAD,PUT,PATCH,POST,DELETE',
    credentials: true,
    allowedHeaders:
      'X-Requested-With, X-HTTP-Method-Override, Content-Type, Accept, Observe',
  });
  app.useGlobalPipes(
    new ValidationPipe({
      transform: true,
      whitelist: true,
      exceptionFactory: (validationErrors: ValidationError[] = []) => {
        const getPrettyClassValidatorErrors = (
          validationErrors: ValidationError[],
          parentProperty = '',
        ): Array<{ property: string; errors: string[] }> => {
          const errors : Array<{ property: string; errors: string[] }> = [];
          const getValidationErrorsRecursively = (
            validationErrors: ValidationError[],
            parentProperty = '',
          ) => {
            for (const error of validationErrors) {
              const propertyPath = parentProperty
                ? `${parentProperty}.${error.property}`
                : error.property;

              if (error.constraints) {
                errors.push({
                  property: propertyPath,
                  errors: Object.values(error.constraints),
                });
              }
              if (error.children?.length) {
                getValidationErrorsRecursively(error.children, propertyPath);
              }
            }
          };
          getValidationErrorsRecursively(validationErrors, parentProperty);
          return errors;
        };
        const errors = getPrettyClassValidatorErrors(validationErrors);
        return new BadRequestException({
          message: 'validation error',
          errors: errors,
        });
      },
    }),
  );
  const swaggerConfig = new DocumentBuilder()
    .setTitle('Music Collection')
    .setDescription('API for demo Music Collection app')
    .setVersion('1.0')
    .addTag('album')
    .build();
  const documentFactory = () => SwaggerModule.createDocument(app, swaggerConfig, {
    include: [AlbumModule, SongModule]
  });
  SwaggerModule.setup('api', app, documentFactory);
  await app.listen(configService.get<number>('app.port') || 5000);
}
bootstrap();
